<?php
/*
Plugin Name: Inline Google Docs
Description: Inline Google Docs allows the user to display contents of his google docs on his pages/posts, using shortcode for markup.
Version: 0.2
Author: Lim Jiunn Haur
Author URI: http://broken-watch.info/
*/

/*  Copyright 2008  Lim Jiunn Haur  (email : fisherman.jiang@broken-watch.info)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// load Zend library to use the Google Docs API
$path = dirname (__FILE__) . "/library";
set_include_path (get_include_path () . PATH_SEPARATOR . $path);

require_once 'Zend/Loader.php';
Zend_Loader::loadClass ('Zend_Http_Client');
Zend_Loader::loadClass ('Zend_Gdata');
Zend_Loader::loadClass ('Zend_Gdata_ClientLogin');
Zend_Loader::loadClass ('Zend_Gdata_Docs');
Zend_Loader::loadClass ('Zend_Gdata_Spreadsheets');
Zend_Loader::loadClass ('Zend_Gdata_Spreadsheets_DocumentQuery');

/**
 * Shortcode handler
 */
function gdocs_display ($atts, $content = NULL){

	try {
	
		switch ($atts['type']){
		
		case 'document':
		
			// Zend HTTP Client
			$httpClient = new Zend_Http_Client ('http://docs.google.com/RawDocContents');
			
			$proxy_host = get_option ('gdocs_proxy_host');
			
			// set proxy settings
			if (isset ($proxy_host) && $proxy_host != ""){
				// set proxy
				$httpClient->setConfig (array (
					'adapter' => 'Zend_Http_Client_Adapter_Proxy',
					'proxy_host' => $proxy_host,
					'proxy_port' => get_option ('gdocs_proxy_port'),
					'proxy_user' => get_option ('gdocs_proxy_user'),
					'proxy_pass' => get_option ('gdocs_proxy_pwd')
				));
			}
			
			// set parameters
			$httpClient->setParameterGet ('action', 'fetch');
			$httpClient->setParameterGet ('justBody', 'true');
			$httpClient->setParameterGet ('revision', '_latest');
			$httpClient->setParameterGet ('editMode', 'false');
			$httpClient->setParameterGet ('docID', $atts['id']);
			
			// conenct to Google
			return "<p>" . $httpClient->request()->getBody() . "</p>";
			
			break;
		
		case 'spreadsheet':
		
			try {
				// get GData client
				$gsClient = gdocs_authenticate (Zend_Gdata_Spreadsheets::AUTH_SERVICE_NAME);
				
				// get feed
				$feed = $gsClient->getFeed("http://spreadsheets.google.com/feeds/list/{$atts['st_id']}/{$atts['wt_id']}/private/full");
				
				// prepare html
				$html = "<table>\r\n";

				// format
				foreach($feed->entries as $entry) {
					$html .= "\t<tr>\r\n\t\t";
					$html .= "<td>" . $entry->title->text . "</td>";
					$cells = split (",", $entry->content->text);
					foreach ($cells as $cell){
						$str = split (": ", $cell);
						$html .= "<td>" . $str[1] . "</td>";
					}
					$html .= "\r\n\t</tr>\r\n";
				}				
				$html .= "</table>\r\n";
				
				return $html;
				
			} catch (Zend_Gdata_App_AuthException $e) {
				// spreadsheets login problem
			}
			
			break;
			
		default:
			
			return NULL;
		
		}
		
	} catch (Zend_Http_Client_Adapter_Exception $e){
		// connnection problem, probably proxy
	}
	
}

/**
 * Hook for options page
 */
function gdocs_options (){

	if (function_exists ('add_options_page')){
		// add an options page that is printed by gdocs_options_setup
		add_options_page ('Inline Google Docs', 'G Docs', 'manage_options', basename(__FILE__), 'gdocs_options_setup');
		
	} 

}

/**
 * Setup and print options page
 */
function gdocs_options_setup (){

?>
	<div class='wrap'>
		<h2>Inline Google Docs</h2>
		<form method='post' action='options.php'>
			<?php wp_nonce_field ('update-options'); ?>
			
			<!-- Begin G Docs options -->
			
			<!-- Login Credentials -->
			<h3>Google Account Login</h3>
			<table cellpadding="2" cellspacing="5">
				<tbody>
					<tr valign="top">
						<th scope="row">Username:</th>
						<td><input type="text" size="40" name="gdocs_user" value="<?php echo get_option ('gdocs_user'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Password:</th>
						<td><input type="password" size="40" name="gdocs_pwd" value="<?php echo get_option ('gdocs_pwd'); ?>" /></td>
					</tr>
				</tbody>
			</table>
<?php

	// login
	try {
		// get Google Docs client
		$gdClient = gdocs_authenticate (Zend_Gdata_Docs::AUTH_SERVICE_NAME);
	} catch (Zend_Gdata_App_AuthException $e) {
		// google docs login problem
		if (get_option ('gdocs_user') == "" || get_option ('gdocs_pwd') == ""){
			echo "<small style='color:red'>Please enter your username and password.</small>";
		}else {
			echo "<small style='color:red'>The plugin was unable to login to the Google service. Did you type the wrong password/username?</small>";
		}
	}

?>			
			<!-- Proxy Settings -->
			<h3>Proxy Settings</h3>
			<table cellpadding="2" cellspacing="5">
				<tbody>
					<tr valign="top">
						<th scope="row">Host:</th>
						<td><input type="text" size"40" name="gdocs_proxy_host" value="<?php echo get_option ('gdocs_proxy_host'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Port:</th>
						<td><input type="text" size"40" name="gdocs_proxy_port" value="<?php echo get_option ('gdocs_proxy_port'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Username:</th>
						<td><input type="text" size"40" name="gdocs_proxy_user" value="<?php echo get_option ('gdocs_proxy_user'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Password:</th>
						<td><input type="text" size"40" name="gdocs_proxy_pwd" value="<?php echo get_option ('gdocs_proxy_pwd'); ?>" /></td>
					</tr>
				</tbody>
			</table>
			
<?php

	// retrieve document list
	try{
	
		if (isset($gdClient)){
			// get documents feed
			$feed = $gdClient->getFeed ("http://docs.google.com/feeds/documents/private/full/-/document");

?>		
			<!-- Document List -->
			<h3>Google Documents</h3>
			<small>Use the corresponding Document ID in your shortcode.<br />
			<span style="color: blue">[gdocs id=<em>doc_id</em> type='document']</span>.</small>
			<table cellpadding="2" cellspacing="5">
				<thead style="text-align:left">
					<tr><th>Title</th><th>Document ID</th></tr>
				</thead>
				<tbody>

<?php if ($feed) foreach ($feed->entries as $entry){ ?>

					<tr><td><?php echo $entry->getTitle(); ?></td><td><?php preg_match ('/^.+%3A(.+)$/', $entry->getId()->getText(), $matches); echo $matches[1]; ?></td></tr>

<?php 
		} 
?>
				</tbody>
			</table>

<?php
			// get spreadsheets feed
			$feed = $gdClient->getFeed ("http://docs.google.com/feeds/documents/private/full/-/spreadsheet");
			
			try {
				// get Google Spreasheets client
				$gsClient = gdocs_authenticate (Zend_Gdata_Spreadsheets::AUTH_SERVICE_NAME);
	
?>
		<!-- Spreadsheet List -->
		<h3>Google Spreadsheets</h3>
		<small>Use the corresponding Spreadsheet ID and Worksheet ID in your shortcode.<br />
		<span style="color: blue">[gdocs st_id=<em>spreadsheet_id</em> wt_id=<em>worksheet_id</em> type='spreadsheet']</span>.</small>

		<table cellpadding="2" cellspacing="5">
			<thead style="text-align:left">
				<tr><th>Title</th><th>Spreadsheet ID</th><th>Worksheets</th></tr>
			</thead>
			<tbody>

<?php if ($feed && isset($gsClient)) foreach ($feed->entries as $entry){ ?>

				<tr><td><?php echo $entry->getTitle(); ?></td><td><?php preg_match ('/^.+%3A(.+)$/', $entry->getId()->getText(), $matches); echo $matches[1]; ?></td><td><?php

					$wtFeed = $gsClient->getFeed ("http://spreadsheets.google.com/feeds/worksheets/{$matches[1]}/private/full");
					
					$count = count($wtFeed->entries);
					for ($i = 0; $i < $count; $i++){
					
						// extract id
						$wtId = split ('/', $wtFeed[$i]->getId()->getText());
						echo $wtId[8];
						
						// if there are more, print comma
						if ($i !== $count-1) echo ", ";
						
					}

?></td></tr>
<?php 
				}
			} catch (Zend_Gdata_App_AuthException $e) {
				// spreadsheets login problem
				// shouldn't come here, since Google Docs login was OK
			
			} 
?>
			</tbody>
		</table>
<?php
		}
	} catch (Zend_Http_Client_Adapter_Exception $e){
		// connnection problem
		echo "<small style='color:red'>Behind a proxy? Input the above settings to connect to Google.</small>";
	}
?>			
			<p class='submit'>
				<input type='submit' name='Submit' value="<?php _e('Save changes') ?>" />
			</p>
			
			<input type="hidden" name="action" value="update" />
			<input type="hidden" name="page_options" value="gdocs_user,gdocs_pwd,gdocs_proxy_host,gdocs_proxy_port,gdocs_proxy_user,gdocs_proxy_pwd" />
			
			
		</form>

	</div>

<?php

}

/**
 * Authenticate user
 */
function gdocs_authenticate ($service){

	// login credentials
	$user = get_option ('gdocs_user');
	$pwd = get_option ('gdocs_pwd');
	
	// create HTTP client
	$httpClient = Zend_Gdata_ClientLogin::getHttpClient($user, $pwd, $service);
	
	$proxy_host = get_option ('gdocs_proxy_host');
	
	if (isset ($proxy_host) && $proxy_host != ""){
		// set proxy
		$httpClient->setConfig (array (
			'adapter' => 'Zend_Http_Client_Adapter_Proxy',
			'proxy_host' => $proxy_host,
			'proxy_port' => get_option ('gdocs_proxy_port'),
			'proxy_user' => get_option ('gdocs_proxy_user'),
			'proxy_pass' => get_option ('gdocs_proxy_pwd')
		));
	}
	
	// create and return Google Docs client
	return new Zend_Gdata($httpClient);

}

/**
 * Install plugin
 */
function gdocs_install (){
	gdocs_set_options ();
}

/**
 * Add configuration options
 */
function gdocs_set_options (){

	// login credentials
	add_option ('gdocs_user');
	add_option ('gdocs_pwd');
	
	// proxy config
	add_option ('gdocs_proxy_host');
	add_option ('gdocs_proxy_port');
	add_option ('gdocs_proxy_user');
	add_option ('gdocs_proxy_pwd');

}

/**
 * Uninstall plugin
 */
function gdocs_uninstall (){
	gdocs_unset_options ();
}

/**
 * Remove configuration options
 */
function gdocs_unset_options (){

	delete_option ('gdocs_user');
	delete_option ('gdocs_pwd');
	delete_option ('gdocs_proxy_host');
	delete_option ('gdocs_proxy_port');
	delete_option ('gdocs_proxy_user');
	delete_option ('gdocs_proxy_pwd');

}

// add hooks
add_action ('admin_menu', 'gdocs_options');
add_shortcode ('gdocs', 'gdocs_display');
register_activation_hook (__FILE__, 'gdocs_install');
register_deactivation_hook (__FILE__, 'gdocs_uninstall');

?>